@@ -1,7 +1,8 @@
 /*
  *      callbacks.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,24 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 669 2006-08-04 12:21:41Z ntrel $
+ * $Id: callbacks.h 3446 2009-01-04 18:30:42Z eht16 $
  */
 
-enum {
-	GEANY_RESPONSE_REPLACE = 1,
-	GEANY_RESPONSE_REPLACE_ALL,
-	GEANY_RESPONSE_REPLACE_SEL,
-	GEANY_RESPONSE_FIND
-};
 
+#include "geany.h" /* necessary for interface.c */
 
-extern gchar current_word[]; //needed for popup menu keybindings access
-
-void
-signal_cb							   (gint sig);
-
-gint
-destroyapp                             (GtkWidget *widget, gpointer gdata);
 
 gboolean
 on_exit_clicked                        (GtkWidget *widget, gpointer gdata);
@@ -63,64 +52,16 @@ void
 on_open1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbutton19_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton6_clicked                 (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
 void
 on_change_font1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton8_clicked                 (GtkToolButton   *toolbutton,
+on_toolbutton_close_clicked            (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton9_clicked                 (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton10_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton15_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton23_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_file_open_dialog_response           (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_notebook1_switch_page               (GtkNotebook     *notebook,
-                                        GtkNotebookPage *page,
-                                        guint            page_num,
-                                        gpointer         user_data);
-
-void
-on_file_save_dialog_response           (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_color_ok_button_clicked             (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_color_cancel_button_clicked         (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_font_cancel_button_clicked          (GtkButton       *button,
+on_toolbutton_close_all_clicked        (GtkAction       *action,
                                         gpointer         user_data);
 
 void
@@ -132,27 +73,9 @@ on_close1_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_font_ok_button_clicked              (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_font_apply_button_clicked           (GtkButton       *button,
-                                        gpointer         user_data);
-
-gboolean
 on_close_all1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-gboolean
-on_editor_button_press_event           (GtkWidget *widget,
-                                        GdkEventButton *event,
-                                        gpointer user_data);
-
-gboolean
-on_tree_view_button_press_event        (GtkWidget *widget,
-                                        GdkEventButton *event,
-                                        gpointer user_data);
-
 void
 on_crlf_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -170,11 +93,7 @@ on_replace_tabs_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton18_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_entry1_activate                     (GtkEntry        *entry,
+on_toolbutton_search_clicked           (GtkAction       *action,
                                         gpointer         user_data);
 
 gboolean
@@ -222,14 +141,6 @@ void
 on_delete1_activate                    (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbutton20_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton21_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
 void
 on_zoom_in1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -239,39 +150,18 @@ on_zoom_out1_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_entry1_changed                      (GtkEditable     *editable,
-                                        gpointer         user_data);
-
+on_toolbar_search_entry_changed        (GtkAction *action,
+                                        const gchar *text,
+                                        gpointer user_data);
 
 void
-on_openfiles_tree_selection_changed    (GtkTreeSelection *selection,
-                                        gpointer data);
-
-void
-on_taglist_tree_selection_changed      (GtkTreeSelection *selection,
-                                        gpointer data);
-
-void
-on_filetype_change                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_to_lower_case1_activate             (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_to_upper_case1_activate             (GtkMenuItem     *menuitem,
+on_toggle_case1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
 on_fullscreen1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-gboolean
-on_window_configure_event              (GtkWidget *widget,
-                                        GdkEventConfigure *event,
-                                        gpointer user_data);
-
 void
 on_show_toolbar1_toggled               (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
@@ -288,10 +178,6 @@ void
 on_invisible1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbutton13_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
 void
 on_find_usage1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -304,19 +190,10 @@ void
 on_goto_tag_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_construct_completion1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-
 void
 on_count_words1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -337,43 +214,6 @@ void
 on_show_color_chooser1_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_build_compile_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_tex_activate                  (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_build_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_make_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_execute_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_arguments_activate            (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_tex_arguments_activate        (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_make_target_dialog_response         (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_make_target_entry_activate          (GtkEntry        *entry,
-                                        gpointer         user_data);
-
 void
 on_find1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -385,26 +225,15 @@ on_find_next1_activate                 (GtkMenuItem     *menuitem,
 void
 on_find_previous1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
-
-void on_find_replace_checkbutton_toggled (GtkToggleButton *togglebutton,
-										  gpointer user_data);
-
 void
-on_find_dialog_response                (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
+on_find_nextsel1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
 void
-on_find_entry_activate                 (GtkEntry        *entry,
+on_find_prevsel1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_replace_dialog_response                (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_replace_entry_activate                 (GtkEntry        *entry,
+on_replace1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
@@ -412,48 +241,39 @@ on_find_in_files1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_find_in_files_dialog_response       (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_new_with_template                   (GtkMenuItem     *menuitem,
+on_toolbutton_new_clicked              (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_new_clicked              (GtkToolButton   *toolbutton,
+on_toolbutton_open_clicked             (GtkAction       *action,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_save_clicked             (GtkAction       *action,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_quit_clicked             (GtkAction       *action,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_preferences_clicked      (GtkAction       *action,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_reload_clicked           (GtkAction       *action,
                                         gpointer         user_data);
 
 void
 on_go_to_line_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_includes_arguments_dialog_response  (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
-                                            gint response,
-                                            gpointer user_data);
-
-void
-on_goto_line_dialog_response           (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_goto_line_entry_activate            (GtkEntry        *entry,
-                                        gpointer         user_data);
-
-
 void
 on_help1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_compile_button_clicked              (GtkToolButton   *toolbutton,
+on_toolbutton_compile_clicked          (GtkAction       *action,
                                         gpointer         user_data);
 
 void
@@ -461,12 +281,7 @@ on_website1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_pref_tools_button_clicked           (GtkButton       *button,
-                                        gpointer         user_data);
-
-
-void
-on_line_breaking1_toggled              (GtkCheckMenuItem *checkmenuitem,
+on_line_wrapping1_toggled              (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
 void
@@ -494,15 +309,16 @@ on_insert_include_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_recent_file_activate                (GtkMenuItem     *menuitem,
+on_custom_date_dialog_response         (GtkDialog *dialog,
+                                        gint response,
+                                        gpointer user_data);
+
+void
+on_custom_date_entry_activate          (GtkEntry        *entry,
                                         gpointer         user_data);
 
 void
-on_file_open_selection_changed         (GtkFileChooser  *filechooser,
-                                        gpointer         user_data);
-
-void
-on_file_open_entry_activate            (GtkEntry        *entry,
+on_insert_date_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
@@ -517,33 +333,12 @@ void
 on_set_file_readonly1_toggled          (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
-void
-on_file_open_check_hidden_toggled      (GtkToggleButton *togglebutton,
-                                        gpointer         user_data);
-
-void
-on_replace1_activate                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_tv_notebook_switch_page             (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
                                         guint            page_num,
                                         gpointer         user_data);
 
-void
-on_openfiles_tree_popup_clicked        (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_taglist_tree_popup_clicked          (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-gboolean
-on_window_key_press_event              (GtkWidget *widget,
-                                        GdkEventKey *event,
-                                        gpointer user_data);
-
 void
 on_help_shortcuts1_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -558,7 +353,7 @@ on_show_line_numbers1_toggled          (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
 void
-on_use_auto_indention1_toggled         (GtkCheckMenuItem *checkmenuitem,
+on_use_auto_indentation1_toggled       (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
 
@@ -572,19 +367,14 @@ on_menu_unfold_all1_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_entry_goto_line_activate            (GtkEntry        *entry,
+on_toolbutton_goto_entry_activate      (GtkAction *action, const gchar *text, gpointer user_data);
+
+void
+on_toolbutton_goto_clicked             (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_goto_clicked             (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_run_button_clicked                  (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_go_to_line1_activate                (GtkMenuItem     *menuitem,
+on_toolbutton_run_clicked              (GtkAction       *action,
                                         gpointer         user_data);
 
 void
@@ -640,3 +430,171 @@ on_menu_increase_indent1_activate      (GtkMenuItem     *menuitem,
 void
 on_menu_decrease_indent1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_menu_toggle_line_commentation1_activate
+                                       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_next_message1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_menu_comments_multiline_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_comments_gpl_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_insert_include_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_insert_date_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_comments_bsd_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_comments_bsd_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_new1_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_open1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_close1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_project_properties1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_project1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_remove_markers1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_load_tags1_activate                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_context_action1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_toggle_all_additional_widgets1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+void
+on_back_activate                       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_forward_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_file1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+gboolean
+on_motion_event                        (GtkWidget       *widget,
+                                        GdkEventMotion  *event,
+                                        gpointer         user_data);
+
+void
+on_tv_notebook_switch_page_after       (GtkNotebook     *notebook,
+                                        GtkNotebookPage *page,
+                                        guint            page_num,
+                                        gpointer         user_data);
+
+void
+on_tabs1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_spaces1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_strip_trailing_spaces1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_page_setup1_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+gboolean
+on_escape_key_press_event              (GtkWidget       *widget,
+                                        GdkEventKey     *event,
+                                        gpointer         user_data);
+
+void
+on_line_breaking1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_replace_spaces_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_previous_message1_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_search1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_close_other_documents1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_reload_configuration1_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_find_document_usage1_activate       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_tabs_and_spaces1_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+void
+on_debug_messages1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_show_white_space1_toggled      (GtkCheckMenuItem *checkmenuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_show_line_endings1_toggled     (GtkCheckMenuItem *checkmenuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_show_indentation_guides1_toggled
+                                        (GtkCheckMenuItem *checkmenuitem,
+                                        gpointer         user_data);
