@@ -2,8 +2,8 @@
  *      navqueue.c - this file is part of Geany, a fast and lightweight IDE
  *
  *      Copyright 2007 Dave Moore <wrex006(at)gmail(dot)com>
- *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: navqueue.c 2779 2008-07-15 14:29:41Z ntrel $
+ * $Id: navqueue.c 3446 2009-01-04 18:30:42Z eht16 $
  */
 
 /*
@@ -35,6 +35,7 @@
 #include "ui_utils.h"
 #include "editor.h"
 #include "navqueue.h"
+#include "toolbar.h"
 
 
 /* for the navigation history queue */
@@ -47,7 +48,7 @@ typedef struct
 static GQueue *navigation_queue;
 static guint nav_queue_pos;
 
-static GtkWidget *navigation_buttons[2];
+static GtkAction *navigation_buttons[2];
 
 
 
@@ -56,8 +57,8 @@ void navqueue_init()
 	navigation_queue = g_queue_new();
 	nav_queue_pos = 0;
 
-	navigation_buttons[0] = lookup_widget(main_widgets.window, "toolbutton_back");
-	navigation_buttons[1] = lookup_widget(main_widgets.window, "toolbutton_forward");
+	navigation_buttons[0] = toolbar_get_action_by_name("NavBack");
+	navigation_buttons[1] = toolbar_get_action_by_name("NavFor");
 }
 
 
@@ -75,23 +76,23 @@ static void adjust_buttons(void)
 {
 	if (g_queue_get_length(navigation_queue) < 2)
 	{
-		gtk_widget_set_sensitive(navigation_buttons[0], FALSE);
-		gtk_widget_set_sensitive(navigation_buttons[1], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[0], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[1], FALSE);
 		return;
 	}
 	if (nav_queue_pos == 0)
 	{
-		gtk_widget_set_sensitive(navigation_buttons[0], TRUE);
-		gtk_widget_set_sensitive(navigation_buttons[1], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[0], TRUE);
+		gtk_action_set_sensitive(navigation_buttons[1], FALSE);
 		return;
 	}
 	/* forward should be sensitive since where not at the start */
-	gtk_widget_set_sensitive(navigation_buttons[1], TRUE);
+	gtk_action_set_sensitive(navigation_buttons[1], TRUE);
 
 	/* back should be sensitive if there's a place to go back to */
 	(nav_queue_pos < g_queue_get_length(navigation_queue) - 1) ?
-		gtk_widget_set_sensitive(navigation_buttons[0], TRUE) :
-			gtk_widget_set_sensitive(navigation_buttons[0], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[0], TRUE) :
+			gtk_action_set_sensitive(navigation_buttons[0], FALSE);
 }
 
 
