@@ -1,8 +1,8 @@
 /*
  *      sciwrappers.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
- *
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
  *      the Free Software Foundation; either version 2 of the License, or
@@ -17,7 +17,15 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 656 2006-08-01 10:35:32Z eht16 $
+ * $Id: sciwrappers.c 3531 2009-01-30 18:53:23Z eht16 $
+ */
+
+/** @file sciwrappers.h
+ * Wrapper functions for the Scintilla editor widget @c SCI_* messages.
+ * You should also check the http://scintilla.org documentation, as it is more detailed.
+ * @note These functions were originally from the cssed project
+ * (http://cssed.sf.net, thanks).
+ * @see ScintillaFuncs::send_message().
  */
 
 #include <string.h>
@@ -27,8 +35,7 @@
 #include "sciwrappers.h"
 #include "utils.h"
 
-
-// stolen from cssed (http://cssed.sf.net), thanks
+#define SSM(s, m, w, l) scintilla_send_message(s, m, w, l)
 
 
 /* line numbers visibility */
@@ -42,7 +49,7 @@ void sci_set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_width)
 		g_snprintf(tmp_str, 15, "_%d%d", len, extra_width);
 		width = SSM(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER, (sptr_t) tmp_str);
 		SSM (sci, SCI_SETMARGINWIDTHN, 0, width);
-		SSM (sci, SCI_SETMARGINSENSITIVEN, 0, FALSE); // use default behaviour
+		SSM (sci, SCI_SETMARGINSENSITIVEN, 0, FALSE); /* use default behaviour */
 	}
 	else
 	{
@@ -71,7 +78,7 @@ void sci_set_mark_long_lines(ScintillaObject * sci, gint type, gint column, cons
 			return;
 		}
 	}
-	SSM(sci, SCI_SETEDGECOLUMN, column - 1, 0);
+	SSM(sci, SCI_SETEDGECOLUMN, column, 0);
 	SSM(sci, SCI_SETEDGECOLOUR, utils_strtod(colour, NULL, TRUE), 0);
 }
 
@@ -143,11 +150,10 @@ gboolean sci_get_visible_eols(ScintillaObject* sci)
 
 void sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set )
 {
-	if(set){
+	if (set)
 		SSM(sci,SCI_SETVIEWWS,SCWS_VISIBLEALWAYS,0);
-	}else{
+	else
 		SSM(sci,SCI_SETVIEWWS,SCWS_INVISIBLE,0);
-	}
 }
 
 gboolean sci_get_visible_white_spaces(ScintillaObject* sci)
@@ -157,12 +163,10 @@ gboolean sci_get_visible_white_spaces(ScintillaObject* sci)
 
 void sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
 {
-	if( set ){
+	if (set)
 		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_WORD,0);
-		SSM(sci, SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_END, 0);
-	}else{
+	else
 		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
-	}
 }
 
 gboolean sci_get_lines_wrapped(ScintillaObject* sci)
@@ -187,14 +191,17 @@ void sci_convert_eols( ScintillaObject* sci, gint eolmode)
 
 void sci_add_text(ScintillaObject* sci, const gchar* text)
 {
-	if( text != NULL ){// if null text is passed to scintilla will segfault
+	if( text != NULL ){ /* if null text is passed to scintilla will segfault */
 		SSM( sci, SCI_ADDTEXT, strlen(text), (sptr_t) text);
 	}
 }
 
+/** Set all text.
+ * @param sci Scintilla widget.
+ * @param text Text. */
 void sci_set_text(ScintillaObject* sci, const gchar* text)
 {
-	if( text != NULL ){// if null text is passed to scintilla will segfault
+	if( text != NULL ){ /* if null text is passed to scintilla will segfault */
 		SSM( sci, SCI_SETTEXT, 0, (sptr_t) text);
 	}
 }
@@ -202,7 +209,7 @@ void sci_set_text(ScintillaObject* sci, const gchar* text)
 
 void sci_add_text_buffer(ScintillaObject* sci, const gchar* text, gint len)
 {
-	if( text != NULL ){// if null text is passed to scintilla will segfault
+	if( text != NULL ){ /* if null text is passed to scintilla will segfault */
 		SSM(sci, SCI_CLEARALL, 0, 0);
 		SSM(sci, SCI_ADDTEXT, len, (sptr_t) text);
 	}
@@ -223,9 +230,10 @@ gboolean sci_can_redo( ScintillaObject* sci )
 
 void sci_undo( ScintillaObject* sci )
 {
-	if( sci_can_undo(sci) ){
+	if( sci_can_undo(sci) )
 		SSM( sci, SCI_UNDO, 0, 0);
-	}else{ // change it to a document function
+	else
+	{ /* change it to a document function */
 
 	}
 }
@@ -233,20 +241,27 @@ void sci_undo( ScintillaObject* sci )
 
 void sci_redo( ScintillaObject* sci )
 {
-	if( sci_can_redo( sci ) ){
+	if( sci_can_redo( sci ) )
 		SSM( sci, SCI_REDO,0,0);
-	}else{// change it to a document function
+	else
+	{ /* change it to a document function */
 
 	}
 }
 
 
+/** Begin grouping a set of edits together as one Undo action.
+ * You must call sci_end_undo_action() after making your edits.
+ * @param sci Scintilla @c GtkWidget. */
 void sci_start_undo_action( ScintillaObject* sci )
 {
 	SSM( sci,SCI_BEGINUNDOACTION,0,0 );
 }
 
 
+/** End grouping a set of edits together as one Undo action.
+ * @param sci Scintilla @c GtkWidget.
+ * @see sci_start_undo_action(). */
 void sci_end_undo_action( ScintillaObject* sci )
 {
 	SSM( sci, SCI_ENDUNDOACTION,0,0);
@@ -271,6 +286,12 @@ void sci_empty_undo_buffer( ScintillaObject* sci )
 }
 
 
+gboolean sci_is_modified(ScintillaObject *sci)
+{
+	return (SSM(sci, SCI_GETMODIFY, 0, 0) != 0);
+}
+
+
 void sci_zoom_in( ScintillaObject* sci )
 {
 	SSM( sci, SCI_ZOOMIN,0,0);
@@ -312,98 +333,136 @@ gboolean sci_is_marker_set_at_line(ScintillaObject* sci, gint line, gint marker)
 {
 	gint state;
 
-	state = SSM( sci, SCI_MARKERGET, line, marker );
-	return(!(state & (1 << marker)));
+	state = SSM( sci, SCI_MARKERGET, line, 0 );
+	return (state & (1 << marker));
 }
 
 
-gboolean sci_marker_next(ScintillaObject* sci, gint line, gint marker_mask)
+/* Returns the line number of the next marker that matches marker_mask, or -1.
+ * marker_mask is a bitor of 1 << marker_index. (See MarkerHandleSet::MarkValue()).
+ * Note: If there is a marker on the line, it returns the same line. */
+gint sci_marker_next(ScintillaObject* sci, gint line, gint marker_mask, gboolean wrap)
 {
 	gint marker_line;
 
 	marker_line = SSM(sci, SCI_MARKERNEXT, line, marker_mask);
-
-	if( marker_line != -1 ){
-		SSM(sci,SCI_GOTOLINE,marker_line,0);
-		return TRUE;
-	}else{
-		return FALSE;
-	}
+	if (wrap && marker_line == -1)
+		marker_line = SSM(sci, SCI_MARKERNEXT, 0, marker_mask);
+	return marker_line;
 }
 
 
-gboolean sci_marker_prev(ScintillaObject* sci, gint line, gint marker_mask)
+/* Returns the line number of the previous marker that matches marker_mask, or -1.
+ * marker_mask is a bitor of 1 << marker_index. (See MarkerHandleSet::MarkValue()).
+ * Note: If there is a marker on the line, it returns the same line. */
+gint sci_marker_previous(ScintillaObject* sci, gint line, gint marker_mask, gboolean wrap)
 {
 	gint marker_line;
 
 	marker_line = SSM(sci, SCI_MARKERPREVIOUS, line, marker_mask);
+	if (wrap && marker_line == -1)
+	{
+		gint len = sci_get_length(sci);
+		gint last_line = sci_get_line_from_position(sci, len - 1);
 
-	if( marker_line != -1 ){
-		SSM(sci,SCI_GOTOLINE,marker_line,0);
-		return TRUE;
-	}else{
-		return FALSE;
+		marker_line = SSM(sci, SCI_MARKERPREVIOUS, last_line, marker_mask);
 	}
+	return marker_line;
 }
 
 
+/** Get line number from position.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 gint sci_get_line_from_position(ScintillaObject* sci, gint position )
 {
 	return SSM(sci, SCI_LINEFROMPOSITION, position, 0);
 }
 
 
+/** Get column number relative to the start of the line that @a position is on.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 gint sci_get_col_from_position(ScintillaObject* sci, gint position )
 {
 	return SSM(sci, SCI_GETCOLUMN, position, 0);
 }
 
 
+/** Get position for the start of @a line.
+ * @param sci Scintilla widget.
+ * @param line Line.
+ * @return Position. */
 gint sci_get_position_from_line(ScintillaObject* sci, gint line )
 {
 	return SSM(sci, SCI_POSITIONFROMLINE, line, 0);
 }
 
 
+/** Get cursor position.
+ * @param sci Scintilla widget.
+ * @return Position. */
 gint sci_get_current_position(ScintillaObject* sci )
 {
 	return SSM(sci, SCI_GETCURRENTPOS, 0, 0);
 }
 
 
-void sci_set_current_position(ScintillaObject* sci, gint position )
+/** Set the cursor position.
+ * @param sci Scintilla widget.
+ * @param position Position.
+ * @param scroll_to_caret Whether to scroll the cursor in view. */
+void sci_set_current_position(ScintillaObject* sci, gint position, gboolean scroll_to_caret)
 {
-	SSM(sci, SCI_GOTOPOS, position, 0);
+	if (scroll_to_caret)
+		SSM(sci, SCI_GOTOPOS, position, 0);
+	else
+	{
+		SSM(sci, SCI_SETCURRENTPOS, position, 0);
+		SSM(sci, SCI_SETANCHOR, position, 0); /* to avoid creation of a selection */
+	}
+	SSM(sci, SCI_CHOOSECARETX, 0, 0);
 }
 
 
-void sci_set_current_line(ScintillaObject* sci, gint line )
+/* Set the cursor line without scrolling the view.
+ * Use sci_goto_line() to also scroll. */
+void sci_set_current_line(ScintillaObject* sci, gint line)
 {
-	SSM(sci, SCI_GOTOLINE, line, 0);
+	gint pos = sci_get_position_from_line(sci, line);
+	sci_set_current_position(sci, pos, FALSE);
 }
 
 
+/** Get total number of lines.
+ * @param sci Scintilla widget. */
 gint sci_get_line_count( ScintillaObject* sci )
 {
 	return SSM(sci, SCI_GETLINECOUNT, 0, 0);
 }
 
 
+/** Set selection start position.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 void sci_set_selection_start(ScintillaObject* sci, gint position)
 {
 	SSM(sci, SCI_SETSELECTIONSTART, position, 0);
 }
 
 
+/** Set selection end position.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 void sci_set_selection_end(ScintillaObject* sci, gint position)
 {
 	SSM(sci, SCI_SETSELECTIONEND, position, 0);
 }
 
 
-gint sci_get_line_end_from_position(ScintillaObject* sci, gint position)
+gint sci_get_line_end_position(ScintillaObject* sci, gint line)
 {
-	return SSM(sci, SCI_GETLINEENDPOSITION, position, 0);
+	return SSM(sci, SCI_GETLINEENDPOSITION, line, 0);
 }
 
 
@@ -431,24 +490,36 @@ void sci_clear(ScintillaObject* sci)
 }
 
 
+/** Get position of selection start.
+ * @param sci Scintilla widget.
+ * @return Position. */
 gint sci_get_selection_start(ScintillaObject* sci)
 {
 	return SSM(sci, SCI_GETSELECTIONSTART,0,0);
 }
 
 
+/** Get position of selection end.
+ * @param sci Scintilla widget.
+ * @return Position. */
 gint sci_get_selection_end(ScintillaObject* sci)
 {
 	return SSM(sci, SCI_GETSELECTIONEND,0,0);
 }
 
 
-void sci_replace_sel(ScintillaObject* sci, gchar* text)
+/** Replace selection.
+ * @param sci Scintilla widget.
+ * @param text Text. */
+void sci_replace_sel(ScintillaObject* sci, const gchar* text)
 {
 	SSM(sci, SCI_REPLACESEL,0, (sptr_t) text);
 }
 
 
+/** Get length of all text.
+ * @param sci Scintilla widget.
+ * @return Length. */
 gint sci_get_length(ScintillaObject* sci)
 {
 	return SSM(sci,SCI_GETLENGTH,0,0);
@@ -461,30 +532,55 @@ gint sci_get_lexer(ScintillaObject* sci)
 }
 
 
+/** Get line length.
+ * @param sci Scintilla widget.
+ * @param line Line number.
+ * @return Length. */
 gint sci_get_line_length(ScintillaObject* sci,gint line)
 {
 	return SSM(sci,SCI_LINELENGTH, line,0);
 }
 
 
-void sci_get_line(ScintillaObject* sci, gint line, gchar* text)
+/* TODO: rename/change to use buffer? Otherwise inconsistent with sci_get_text*(). */
+/** Get line contents.
+ * @param sci Scintilla widget.
+ * @param line_num Line number.
+ * @return A @c NULL-terminated copy of the line text. */
+gchar *sci_get_line(ScintillaObject* sci, gint line_num)
 {
-	SSM(sci,SCI_GETLINE, line, (sptr_t) text);
+	gint len = sci_get_line_length(sci, line_num);
+	gchar *linebuf = g_malloc(len + 1);
+
+	SSM(sci, SCI_GETLINE, line_num, (sptr_t) linebuf);
+	linebuf[len] = '\0';
+	return linebuf;
 }
 
-// the last char will be null terminated
+
+/** Get all text.
+ * @param sci Scintilla widget.
+ * @param len Length of @a text buffer, usually sci_get_length() + 1.
+ * @param text Text buffer; must be allocated @a len + 1 bytes for null-termination. */
 void sci_get_text(ScintillaObject* sci, gint len, gchar* text)
 {
 	SSM( sci, SCI_GETTEXT, len, (sptr_t) text );
 }
 
 
+/** Get selected text.
+ * @param sci Scintilla widget.
+ * @param text Text buffer; must be allocated sci_get_selected_text_length() + 1 bytes
+ * for null-termination. */
 void sci_get_selected_text(ScintillaObject* sci, gchar* text)
 {
 	SSM( sci, SCI_GETSELTEXT, 0, (sptr_t) text);
 }
 
 
+/** Get selected text length.
+ * @param sci Scintilla widget.
+ * @return Length. */
 gint sci_get_selected_text_length(ScintillaObject* sci)
 {
 	return SSM( sci, SCI_GETSELTEXT, 0, 0);
@@ -493,7 +589,7 @@ gint sci_get_selected_text_length(ScintillaObject* sci)
 
 gint sci_get_position_from_xy(ScintillaObject* sci, gint x, gint y, gboolean nearby)
 {
-	// for nearby return -1 if there is no character near to the x,y point.
+	/* for nearby return -1 if there is no character near to the x,y point. */
 	return SSM(sci, (nearby) ? SCI_POSITIONFROMPOINTCLOSE : SCI_POSITIONFROMPOINT, x, y);
 }
 
@@ -505,13 +601,19 @@ void sci_get_xy_from_position(ScintillaObject* sci,gint pos, gint* x, gint* y)
 }
 
 
-/* folding */
+/** Check if a line is visible (folding may have hidden it).
+ * @param sci Scintilla widget.
+ * @param line Line number.
+ * @return Whether @a line will be drawn on the screen. */
 gboolean sci_get_line_is_visible(ScintillaObject* sci, gint line)
 {
 	return SSM(sci,SCI_GETLINEVISIBLE, line,0);
 }
 
 
+/** Make the @a line visible (folding may have hidden it).
+ * @param sci Scintilla widget.
+ * @param line Line number. */
 void sci_ensure_line_is_visible(ScintillaObject* sci, gint line)
 {
 	SSM(sci,SCI_ENSUREVISIBLE,line,0);
@@ -524,6 +626,20 @@ gint sci_get_fold_level(ScintillaObject* sci, gint line)
 }
 
 
+/* Get the next line after start_line with fold level <= level */
+gint sci_get_last_child(ScintillaObject* sci, gint start_line, gint level)
+{
+	return SSM( sci, SCI_GETLASTCHILD, start_line, level);
+}
+
+
+/* Get the line number of the fold point before start_line, or -1 if there isn't one */
+gint sci_get_fold_parent(ScintillaObject* sci, gint start_line)
+{
+	return SSM( sci, SCI_GETFOLDPARENT, start_line, 0);
+}
+
+
 void sci_toggle_fold(ScintillaObject* sci, gint line)
 {
 	SSM( sci, SCI_TOGGLEFOLD, line, 1);
@@ -560,24 +676,28 @@ gint sci_get_end_styled(ScintillaObject * sci)
 }
 
 
-gint sci_get_line_end_styled(ScintillaObject * sci, gint end_styled)
-{
-	return SSM(sci,SCI_LINEFROMPOSITION, end_styled,0);
-}
-
-
 void sci_set_tab_width(ScintillaObject * sci, gint width)
 {
 	SSM(sci, SCI_SETTABWIDTH, width, 0);
 }
 
 
+/** Get display tab width (this is not indent width, see GeanyIndentPrefs).
+ * @param sci Scintilla widget.
+ * @return Width.
+ *
+ * @since 0.15
+ **/
 gint sci_get_tab_width(ScintillaObject * sci)
 {
 	return SSM(sci, SCI_GETTABWIDTH, 0, 0);
 }
 
 
+/** Get character.
+ * @param sci Scintilla widget.
+ * @param pos Position.
+ * @return Char. */
 gchar sci_get_char_at(ScintillaObject *sci, gint pos)
 {
 	return (gchar) SSM(sci, SCI_GETCHARAT, pos, 0);
@@ -589,10 +709,9 @@ void sci_set_savepoint(ScintillaObject *sci)
 	SSM(sci, SCI_SETSAVEPOINT, 0, 0);
 }
 
-
-void sci_set_indentionguides(ScintillaObject *sci, gboolean enable)
+void sci_set_indentation_guides(ScintillaObject *sci, gint mode)
 {
-	SSM(sci, SCI_SETINDENTATIONGUIDES, enable, 0);
+	SSM(sci, SCI_SETINDENTATIONGUIDES, mode, 0);
 }
 
 
@@ -602,8 +721,13 @@ void sci_use_popup(ScintillaObject *sci, gboolean enable)
 }
 
 
-// you can also call this has_selection
-gboolean sci_can_copy(ScintillaObject *sci)
+/** Check if there's a selection.
+ * @param sci Scintilla widget.
+ * @return Whether a selection is present.
+ *
+ * @since 0.15
+ **/
+gboolean sci_has_selection(ScintillaObject *sci)
 {
 	if (SSM(sci, SCI_GETSELECTIONEND,0,0) - SSM(sci, SCI_GETSELECTIONSTART,0,0))
 		return TRUE;
@@ -612,9 +736,9 @@ gboolean sci_can_copy(ScintillaObject *sci)
 }
 
 
-void sci_goto_pos(ScintillaObject *sci, gint pos, gboolean ensure_visibility)
+void sci_goto_pos(ScintillaObject *sci, gint pos, gboolean unfold)
 {
-	if (ensure_visibility) SSM(sci,SCI_ENSUREVISIBLE,SSM(sci, SCI_LINEFROMPOSITION, pos, 0),0);
+	if (unfold) SSM(sci,SCI_ENSUREVISIBLE,SSM(sci, SCI_LINEFROMPOSITION, pos, 0),0);
 	SSM(sci, SCI_GOTOPOS, pos, 0);
 }
 
@@ -631,12 +755,26 @@ void sci_set_anchor(ScintillaObject *sci, gint pos)
 }
 
 
+/** Scroll the cursor in view.
+ * @param sci Scintilla widget. */
 void sci_scroll_caret(ScintillaObject *sci)
 {
 	SSM(sci, SCI_SCROLLCARET, 0, 0);
 }
 
 
+void sci_scroll_lines(ScintillaObject *sci, gint lines)
+{
+	SSM(sci, SCI_LINESCROLL, 0, lines);
+}
+
+
+void sci_scroll_columns(ScintillaObject *sci, gint columns)
+{
+	SSM(sci, SCI_LINESCROLL, columns, 0);
+}
+
+
 gint sci_search_next(ScintillaObject *sci, gint flags, const gchar *text)
 {
 	return SSM(sci, SCI_SEARCHNEXT, flags, (sptr_t) text );
@@ -662,34 +800,23 @@ void sci_set_font(ScintillaObject *sci, gint style, const gchar* font, gint size
 }
 
 
-void sci_goto_line(ScintillaObject *sci, gint line, gboolean ensure_visibility)
+void sci_goto_line(ScintillaObject *sci, gint line, gboolean unfold)
 {
-	if (ensure_visibility) SSM(sci,SCI_ENSUREVISIBLE,line,0);
+	if (unfold) SSM(sci, SCI_ENSUREVISIBLE, line, 0);
 	SSM(sci, SCI_GOTOLINE, line, 0);
 }
 
 
-void sci_goto_line_scroll(ScintillaObject *sci, gint line, gdouble percent_of_view)
-{
-	gint vis1, los, delta;
-
-	sci_goto_line(sci, line, TRUE);
-	// sci 'visible line' != file line number
-	vis1 = SSM(sci, SCI_GETFIRSTVISIBLELINE, 0, 0);
-	vis1 = SSM(sci, SCI_DOCLINEFROMVISIBLE, vis1, 0);
-	los = SSM(sci, SCI_LINESONSCREEN, 0, 0);
-	delta = (line - vis1) - los * percent_of_view;
-	sci_scroll_lines(sci, delta);
-	sci_scroll_caret(sci); //ensure visible, in case of excessive folding/wrapping
-}
-
-
 void sci_marker_delete_all(ScintillaObject *sci, gint marker)
 {
 	SSM(sci, SCI_MARKERDELETEALL, marker, 0);
 }
 
 
+/** Get style ID for @a position.
+ * @param sci Scintilla widget.
+ * @param position Position.
+ * @return Style ID. */
 gint sci_get_style_at(ScintillaObject *sci, gint position)
 {
 	return SSM(sci, SCI_GETSTYLEAT, position, 0);
@@ -714,6 +841,11 @@ void sci_clear_cmdkey(ScintillaObject *sci, gint key)
 }
 
 
+/** Get text between @a start and @a end.
+ * @param sci Scintilla widget.
+ * @param start Start.
+ * @param end End.
+ * @param text Text will be zero terminated and must be allocated (end - start + 1) bytes. */
 void sci_get_text_range(ScintillaObject *sci, gint start, gint end, gchar *text)
 {
 	struct TextRange tr;
@@ -736,13 +868,17 @@ void sci_selection_duplicate(ScintillaObject *sci)
 }
 
 
+/** Insert text.
+ * @param sci Scintilla widget.
+ * @param pos Position.
+ * @param text Text. */
 void sci_insert_text(ScintillaObject *sci, gint pos, const gchar *text)
 {
 	SSM(sci, SCI_INSERTTEXT, pos, (sptr_t) text);
 }
 
 
-void sci_grap_focus(ScintillaObject *sci)
+void sci_grab_focus(ScintillaObject *sci)
 {
 	SSM(sci, SCI_GRABFOCUS, 0, 0);
 }
@@ -774,7 +910,7 @@ void sci_target_end(ScintillaObject *sci, gint end)
 
 gint sci_target_replace(ScintillaObject *sci, const gchar *text, gboolean regex)
 {
-	return SSM(sci, (regex) ? SCI_REPLACETARGETRE : SCI_REPLACETARGET, -1, (sptr_t) text);
+	return SSM(sci, (regex) ? SCI_REPLACETARGETRE : SCI_REPLACETARGET, (uptr_t) -1, (sptr_t) text);
 }
 
 
@@ -783,11 +919,6 @@ void sci_set_keywords(ScintillaObject *sci, gint k, gchar *text)
 	SSM(sci, SCI_SETKEYWORDS, k, (sptr_t) text);
 }
 
-void sci_scroll_lines(ScintillaObject *sci, gint lines)
-{
-	SSM(sci, SCI_LINESCROLL, 0, lines);
-}
-
 void sci_set_readonly(ScintillaObject *sci, gboolean readonly)
 {
 	SSM(sci, SCI_SETREADONLY, readonly, 0);
@@ -798,38 +929,51 @@ gboolean sci_get_readonly(ScintillaObject *sci)
 	return SSM(sci, SCI_GETREADONLY, 0, 0);
 }
 
-// a simple convenience function to not have SSM() in the outside of this file
- void sci_cmd(ScintillaObject * sci, gint cmd)
+/** A simple convenience function for sending Scintilla commands without any parameters.
+ * @param sci The Scintilla @c GtkWidget.
+ * @param cmd @c SCI_COMMAND.
+ * @see http://scintilla.org for the documentation.
+ *
+ *  @since 0.16
+ */
+void sci_send_command(ScintillaObject * sci, gint cmd)
 {
 	SSM(sci, cmd, 0, 0);
 }
 
-gint sci_get_current_line(ScintillaObject *sci, gint pos)
+
+/** Get current line number.
+ * @param sci Scintilla widget.
+ * @return Line number. */
+gint sci_get_current_line(ScintillaObject *sci)
 {
-	if (pos >= 0)
-	{
-		return SSM(sci, SCI_LINEFROMPOSITION, pos, 0);
-	}
-	else
-	{
-		return SSM(sci, SCI_LINEFROMPOSITION, SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
-	}
+	return SSM(sci, SCI_LINEFROMPOSITION, SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
 }
 
-// get number of lines partially or fully selected
+/* Get number of lines partially or fully selected.
+ * Returns 1 if there is a partial selection on the same line.
+ * Returns 2 if a whole line is selected including the line break char(s). */
 gint sci_get_lines_selected(ScintillaObject *sci)
 {
 	gint start = SSM(sci, SCI_GETSELECTIONSTART, 0, 0);
 	gint end = SSM(sci, SCI_GETSELECTIONEND, 0, 0);
 
 	if (start == end)
-		return 0; // no selection
+		return 0; /* no selection */
 
 	return SSM(sci, SCI_LINEFROMPOSITION, end, 0) - SSM(sci, SCI_LINEFROMPOSITION, start, 0) + 1;
 }
 
+gint sci_get_first_visible_line(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETFIRSTVISIBLELINE, 0, 0);
+}
+
+
 void sci_set_styling(ScintillaObject *sci, gint len, gint style)
 {
+	if (len < 0 || style < 0) return;
+
 	SSM(sci, SCI_SETSTYLING, len, style);
 }
 
@@ -838,6 +982,49 @@ void sci_start_styling(ScintillaObject *sci, gint pos, gint mask)
 	SSM(sci, SCI_STARTSTYLING, pos, mask);
 }
 
+gint sci_indicator_get(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETINDICATORCURRENT, 0, 0);
+}
+
+/**
+ *  Set the current indicator. This is necessary to define an indicator for a range of text or
+ *  clearing indicators for a range of text.
+ *
+ *  @param sci Scintilla widget.
+ *  @param indic The indicator number to set.
+ *
+ *  @see sci_indicator_clear
+ *
+ *  @since 0.16
+ */
+void sci_indicator_set(ScintillaObject *sci, gint indic)
+{
+	SSM(sci, SCI_SETINDICATORCURRENT, indic, 0);
+}
+
+void sci_indicator_fill(ScintillaObject *sci, gint pos, gint len)
+{
+	SSM(sci, SCI_INDICATORFILLRANGE, pos, len);
+}
+
+/**
+ *  Clear a range of text from the currently set indicator.
+ *  Starting at @a pos, @a len characters long.
+ *  In order to make this function properly, you need to set the current indicator before with
+ *  @ref sci_indicator_set().
+ *
+ *  @param sci Scintilla widget.
+ *  @param pos Starting position.
+ *  @param len Length.
+ *
+ *  @since 0.16
+ */
+void sci_indicator_clear(ScintillaObject *sci, gint pos, gint len)
+{
+	SSM(sci, SCI_INDICATORCLEARRANGE, pos, len);
+}
+
 void sci_select_all(ScintillaObject *sci)
 {
 	SSM(sci, SCI_SELECTALL, 0, 0);
@@ -847,3 +1034,109 @@ gint sci_get_line_indent_position(ScintillaObject *sci, gint line)
 {
 	return SSM(sci, SCI_GETLINEINDENTPOSITION, line, 0);
 }
+
+void sci_set_autoc_max_height(ScintillaObject *sci, gint val)
+{
+	SSM(sci, SCI_AUTOCSETMAXHEIGHT, val, 0);
+}
+
+/** Find a matching brace at @a pos.
+ * @param sci Scintilla widget.
+ * @param pos Position.
+ * @return Matching brace position.
+ *
+ * @since 0.15
+ **/
+gint sci_find_matching_brace(ScintillaObject *sci, gint pos)
+{
+	return SSM(sci, SCI_BRACEMATCH, pos, 0);
+}
+
+gint sci_get_overtype(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETOVERTYPE, 0, 0);
+}
+
+void sci_set_tab_indents(ScintillaObject *sci, gboolean set)
+{
+	SSM(sci, SCI_SETTABINDENTS, set, 0);
+}
+
+void sci_set_use_tabs(ScintillaObject *sci, gboolean set)
+{
+	SSM(sci, SCI_SETUSETABS, set, 0);
+}
+
+gint sci_get_pos_at_line_sel_start(ScintillaObject *sci, gint line)
+{
+	return SSM(sci, SCI_GETLINESELSTARTPOSITION, line, 0);
+}
+
+gint sci_get_pos_at_line_sel_end(ScintillaObject *sci, gint line)
+{
+	return SSM(sci, SCI_GETLINESELENDPOSITION, line, 0);
+}
+
+/** Get selection mode.
+ * @param sci Scintilla widget.
+ * @return Selection mode. */
+gint sci_get_selection_mode(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETSELECTIONMODE, 0, 0);
+}
+
+/** Set selection mode.
+ * @param sci Scintilla widget.
+ * @param mode Mode. */
+void sci_set_selection_mode(ScintillaObject *sci, gint mode)
+{
+	SSM(sci, SCI_SETSELECTIONMODE, mode, 0);
+}
+
+void sci_set_scrollbar_mode(ScintillaObject *sci, gboolean visible)
+{
+	SSM(sci, SCI_SETHSCROLLBAR, visible, 0);
+	SSM(sci, SCI_SETVSCROLLBAR, visible, 0);
+}
+
+void sci_set_line_indentation(ScintillaObject *sci, gint line, gint indent)
+{
+	SSM(sci, SCI_SETLINEINDENTATION, line, indent);
+}
+
+int sci_get_line_indentation(ScintillaObject *sci, gint line)
+{
+	return SSM(sci, SCI_GETLINEINDENTATION, line, 0);
+}
+
+void sci_set_caret_policy_x(ScintillaObject *sci, gint policy, gint slop)
+{
+	SSM(sci, SCI_SETXCARETPOLICY, policy, slop);
+}
+
+void sci_set_caret_policy_y(ScintillaObject *sci, gint policy, gint slop)
+{
+	SSM(sci, SCI_SETYCARETPOLICY, policy, slop);
+}
+
+void sci_set_scroll_stop_at_last_line(ScintillaObject* sci, gboolean set)
+{
+	SSM(sci, SCI_SETENDATLASTLINE, set, 0);
+}
+
+void sci_cancel(ScintillaObject *sci)
+{
+	SSM( sci, SCI_CANCEL, 0, 0);
+}
+
+gint sci_get_target_end(ScintillaObject *sci)
+{
+	return SSM( sci, SCI_GETTARGETEND, 0, 0);
+}
+
+gint sci_get_position_after(ScintillaObject *sci, gint start)
+{
+	return SSM(sci, SCI_POSITIONAFTER, start, 0);
+}
+
+
