@@ -1,8 +1,8 @@
 /*
  *      treeviews.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: treeviews.c 3056 2008-10-07 18:51:50Z eht16 $
+ * $Id: treeviews.c 3571 2009-02-10 21:11:25Z eht16 $
  */
 
 /*
@@ -44,6 +44,14 @@
 
 SidebarTreeviews tv;
 
+static struct
+{
+	GtkWidget *close;
+	GtkWidget *save;
+	GtkWidget *reload;
+}
+doc_items = {NULL, NULL, NULL};
+
 enum
 {
 	TREEVIEW_SYMBOL = 0,
@@ -54,38 +62,31 @@ enum
 {
 	OPENFILES_ACTION_REMOVE = 0,
 	OPENFILES_ACTION_SAVE,
-	OPENFILES_ACTION_RELOAD,
-	OPENFILES_ACTION_HIDE,
-	OPENFILES_ACTION_HIDE_ALL,
-	SYMBOL_ACTION_SORT_BY_NAME,
-	SYMBOL_ACTION_SORT_BY_APPEARANCE,
-	SYMBOL_ACTION_HIDE,
-	SYMBOL_ACTION_HIDE_ALL
+	OPENFILES_ACTION_RELOAD
 };
 
-typedef struct
+/* documents tree model columns */
+enum
 {
-	GtkWidget *documents_fullpath;
-	GtkWidget *documents_show_symbols;
-	GtkWidget *documents_show_documents;
-	GtkWidget *symbols_show_symbols;
-	GtkWidget *symbols_show_documents;
-} menu_items;
-static menu_items mi;
+	DOCUMENTS_SHORTNAME,	/* dirname for parents, basename for children */
+	DOCUMENTS_DOCUMENT,
+	DOCUMENTS_COLOR,
+	DOCUMENTS_FILENAME		/* full filename */
+};
 
-static GtkListStore	*store_openfiles;
+static GtkTreeStore	*store_openfiles;
 static GtkWidget *tag_window;	/* scrolled window that holds the symbol list GtkTreeView */
 
 /* callback prototypes */
-static void on_taglist_tree_popup_clicked(GtkMenuItem *menuitem, gpointer user_data);
 static void on_openfiles_tree_selection_changed(GtkTreeSelection *selection, gpointer data);
 static void on_openfiles_document_action(GtkMenuItem *menuitem, gpointer user_data);
-static void on_openfiles_hide_item_clicked(GtkMenuItem *menuitem, gpointer user_data);
 static gboolean on_taglist_tree_selection_changed(GtkTreeSelection *selection);
-static gboolean on_treeviews_button_press_event(GtkWidget *widget, GdkEventButton *event,
-																			gpointer user_data);
+static gboolean on_symbols_button_press_event(GtkWidget *widget, GdkEventButton *event,
+		gpointer user_data);
+static gboolean on_documents_button_release_event(GtkWidget *widget, GdkEventButton *event,
+		gpointer user_data);
 static gboolean on_treeviews_key_press_event(GtkWidget *widget, GdkEventKey *event,
-																			gpointer user_data);
+		gpointer user_data);
 static void on_list_document_activate(GtkCheckMenuItem *item, gpointer user_data);
 static void on_list_symbol_activate(GtkCheckMenuItem *item, gpointer user_data);
 
@@ -96,7 +97,6 @@ static void prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
 	GtkCellRenderer *text_renderer, *icon_renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
-	PangoFontDescription *pfd;
 
 	text_renderer = gtk_cell_renderer_text_new();
 	icon_renderer = gtk_cell_renderer_pixbuf_new();
@@ -114,25 +114,28 @@ static void prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), FALSE);
 
-	pfd = pango_font_description_from_string(interface_prefs.tagbar_font);
-	gtk_widget_modify_font(tree, pfd);
-	pango_font_description_free(pfd);
+	ui_widget_modify_font_from_string(tree, interface_prefs.tagbar_font);
 
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
 	g_signal_connect(tree, "button-press-event",
-					G_CALLBACK(on_treeviews_button_press_event), GINT_TO_POINTER(TREEVIEW_SYMBOL));
+		G_CALLBACK(on_symbols_button_press_event), NULL);
 	g_signal_connect(tree, "key-press-event",
-					G_CALLBACK(on_treeviews_key_press_event), GINT_TO_POINTER(TREEVIEW_SYMBOL));
+		G_CALLBACK(on_treeviews_key_press_event), GINT_TO_POINTER(TREEVIEW_SYMBOL));
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(tree), FALSE);
 
-#if GTK_CHECK_VERSION(2, 12, 0)
-	gtk_tree_view_set_show_expanders(GTK_TREE_VIEW(tree), interface_prefs.show_symbol_list_expanders);
-	if (! interface_prefs.show_symbol_list_expanders)
-		gtk_tree_view_set_level_indentation(GTK_TREE_VIEW(tree), 10);
-#endif
+	if (gtk_check_version(2, 12, 0) == NULL)
+	{
+		g_object_set(tree, "show-expanders", interface_prefs.show_symbol_list_expanders, NULL);
+		if (! interface_prefs.show_symbol_list_expanders)
+			g_object_set(tree, "level-indentation", 10, NULL);
+		/* Tooltips */
+		g_object_set(tree,
+			"has-tooltip", TRUE,
+			"tooltip-column", SYMBOLS_COLUMN_TOOLTIP, NULL);
+	}
 
 	/* selection handling */
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
@@ -147,7 +150,7 @@ on_default_tag_tree_button_press_event(GtkWidget *widget, GdkEventButton *event,
 {
 	if (event->button == 3)
 	{
-		on_treeviews_button_press_event(widget, event, GINT_TO_POINTER(TREEVIEW_SYMBOL));
+		on_symbols_button_press_event(widget, event, NULL);
 	}
 	return FALSE;
 }
@@ -190,7 +193,7 @@ void treeviews_update_tag_list(GeanyDocument *doc, gboolean update)
 		if (doc->priv->tag_tree == NULL)
 		{
 			doc->priv->tag_store = gtk_tree_store_new(
-				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT);
+				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
 			doc->priv->tag_tree = gtk_tree_view_new();
 			prepare_taglist(doc->priv->tag_tree, doc->priv->tag_store);
 			gtk_widget_show(doc->priv->tag_tree);
@@ -211,62 +214,31 @@ void treeviews_update_tag_list(GeanyDocument *doc, gboolean update)
 }
 
 
-#if GTK_CHECK_VERSION(2, 12, 0)
-gboolean on_treeviews_tooltip_queried(GtkWidget *widget, gint x, gint y, gboolean keyboard_mode,
-									  GtkTooltip *tooltip, gpointer user_data)
-{
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-
-	if (gtk_tree_view_get_tooltip_context(
-			GTK_TREE_VIEW(widget), &x, &y, keyboard_mode, &model, NULL, &iter))
-	{
-		gchar *file_name = NULL;
-
-		gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), &iter, 3, &file_name, -1);
-		if (file_name != NULL)
-		{
-			gtk_tooltip_set_text(tooltip, file_name);
-			g_free(file_name);
-			return TRUE;
-		}
-	}
-
-	return FALSE;
-}
-#endif
-
-
 /* does some preparing things to the open files list widget */
 static void prepare_openfiles(void)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
-	PangoFontDescription *pfd;
 	GtkTreeSortable *sortable;
 
-	tv.tree_openfiles = lookup_widget(main_widgets.window, "treeview6");
+	tv.tree_openfiles = ui_lookup_widget(main_widgets.window, "treeview6");
 
 	/* store the short filename to show, and the index as reference,
 	 * the colour (black/red/green) and the full name for the tooltip */
-#if GTK_CHECK_VERSION(2, 12, 0)
-	store_openfiles = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_POINTER, GDK_TYPE_COLOR, G_TYPE_STRING);
-#else
-	store_openfiles = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_POINTER, GDK_TYPE_COLOR);
-#endif
+	store_openfiles = gtk_tree_store_new(4, G_TYPE_STRING, G_TYPE_POINTER, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tv.tree_openfiles), GTK_TREE_MODEL(store_openfiles));
 	g_object_unref(store_openfiles);
 
 	/* set policy settings for the scolledwindow around the treeview again, because glade
 	 * doesn't keep the settings */
 	gtk_scrolled_window_set_policy(
-			GTK_SCROLLED_WINDOW(lookup_widget(main_widgets.window, "scrolledwindow7")),
+			GTK_SCROLLED_WINDOW(ui_lookup_widget(main_widgets.window, "scrolledwindow7")),
 			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Documents"), renderer,
-															"text", 0, "foreground-gdk", 2, NULL);
+		"text", DOCUMENTS_SHORTNAME, "foreground-gdk", DOCUMENTS_COLOR, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tv.tree_openfiles), column);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv.tree_openfiles), FALSE);
 
@@ -274,21 +246,16 @@ static void prepare_openfiles(void)
 
 	/* sort opened filenames in the store_openfiles treeview */
 	sortable = GTK_TREE_SORTABLE(GTK_TREE_MODEL(store_openfiles));
-	gtk_tree_sortable_set_sort_column_id(sortable, 0, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id(sortable, DOCUMENTS_SHORTNAME, GTK_SORT_ASCENDING);
 
-	pfd = pango_font_description_from_string(interface_prefs.tagbar_font);
-	gtk_widget_modify_font(tv.tree_openfiles, pfd);
-	pango_font_description_free(pfd);
+	ui_widget_modify_font_from_string(tv.tree_openfiles, interface_prefs.tagbar_font);
 
-#if GTK_CHECK_VERSION(2, 12, 0)
 	/* GTK 2.12 tooltips */
-	gtk_widget_set_has_tooltip(tv.tree_openfiles, TRUE);
-	g_signal_connect(tv.tree_openfiles, "query-tooltip",
-						G_CALLBACK(on_treeviews_tooltip_queried), NULL);
-#endif
+	if (gtk_check_version(2, 12, 0) == NULL)
+		g_object_set(tv.tree_openfiles, "has-tooltip", TRUE, "tooltip-column", DOCUMENTS_FILENAME, NULL);
 
-	g_signal_connect(tv.tree_openfiles, "button-press-event",
-			G_CALLBACK(on_treeviews_button_press_event), GINT_TO_POINTER(TREEVIEW_OPENFILES));
+	g_signal_connect(tv.tree_openfiles, "button-release-event",
+		G_CALLBACK(on_documents_button_release_event), NULL);
 
 	/* selection handling */
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
@@ -297,44 +264,135 @@ static void prepare_openfiles(void)
 }
 
 
-/* Also sets documents[idx]->iter.
+/* iter should be toplevel */
+static gboolean find_tree_iter_dir(GtkTreeIter *iter, const gchar *dir)
+{
+	GeanyDocument *doc;
+	gchar *name;
+
+	if (utils_str_equal(dir, "."))
+		dir = GEANY_STRING_UNTITLED;
+
+	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, DOCUMENTS_DOCUMENT, &doc, -1);
+	g_return_val_if_fail(!doc, FALSE);
+
+	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, DOCUMENTS_SHORTNAME, &name, -1);
+	return utils_str_equal(name, dir);
+}
+
+
+static GtkTreeIter *get_doc_parent(GeanyDocument *doc)
+{
+	gchar *dirname;
+	static GtkTreeIter parent;
+	GtkTreeModel *model = GTK_TREE_MODEL(store_openfiles);
+
+	dirname = g_path_get_dirname(DOC_FILENAME(doc));
+
+	if (gtk_tree_model_get_iter_first(model, &parent))
+	{
+		do
+		{
+			if (find_tree_iter_dir(&parent, dirname))
+			{
+				g_free(dirname);
+				return &parent;
+			}
+		}
+		while (gtk_tree_model_iter_next(model, &parent));
+	}
+	/* no match, add dir parent */
+	gtk_tree_store_append(store_openfiles, &parent, NULL);
+	gtk_tree_store_set(store_openfiles, &parent, DOCUMENTS_SHORTNAME,
+		doc->file_name ? dirname : GEANY_STRING_UNTITLED, -1);
+
+	g_free(dirname);
+	return &parent;
+}
+
+
+/* Also sets doc->priv->iter.
  * This is called recursively in treeviews_openfiles_update_all(). */
 void treeviews_openfiles_add(GeanyDocument *doc)
 {
 	GtkTreeIter *iter = &doc->priv->iter;
+	GtkTreeIter *parent = get_doc_parent(doc);
+	gchar *basename;
+	const GdkColor *color = document_get_status_color(doc);
 
-	gtk_list_store_append(store_openfiles, iter);
-	treeviews_openfiles_update(doc);
+	gtk_tree_store_append(store_openfiles, iter, parent);
+
+	/* check if new parent */
+	if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store_openfiles), parent) == 1)
+	{
+		GtkTreePath *path;
+
+		/* expand parent */
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(store_openfiles), parent);
+		gtk_tree_view_expand_row(GTK_TREE_VIEW(tv.tree_openfiles), path, TRUE);
+		gtk_tree_path_free(path);
+	}
+	basename = g_path_get_basename(DOC_FILENAME(doc));
+	gtk_tree_store_set(store_openfiles, iter,
+		DOCUMENTS_SHORTNAME, basename, DOCUMENTS_DOCUMENT, doc, DOCUMENTS_COLOR, color,
+		DOCUMENTS_FILENAME, DOC_FILENAME(doc), -1);
+	g_free(basename);
+}
+
+
+static void openfiles_remove(GeanyDocument *doc)
+{
+	GtkTreeIter *iter = &doc->priv->iter;
+	GtkTreeIter parent;
+
+	if (gtk_tree_model_iter_parent(GTK_TREE_MODEL(store_openfiles), &parent, iter) &&
+		gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store_openfiles), &parent) == 1)
+		gtk_tree_store_remove(store_openfiles, &parent);
+	else
+		gtk_tree_store_remove(store_openfiles, iter);
 }
 
 
 void treeviews_openfiles_update(GeanyDocument *doc)
 {
-	gchar *basename;
-	GdkColor *color = document_get_status_color(doc);
+	GtkTreeIter *iter = &doc->priv->iter;
+	gchar *fname;
 
-	if (interface_prefs.sidebar_openfiles_fullpath)
-		basename = DOC_FILENAME(doc);
+	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, DOCUMENTS_FILENAME, &fname, -1);
+
+	if (utils_str_equal(fname, DOC_FILENAME(doc)))
+	{
+		/* just update color */
+		const GdkColor *color = document_get_status_color(doc);
+
+		gtk_tree_store_set(store_openfiles, iter, DOCUMENTS_COLOR, color, -1);
+	}
 	else
-		basename = g_path_get_basename(DOC_FILENAME(doc));
-	gtk_list_store_set(store_openfiles, &doc->priv->iter,
-#if GTK_CHECK_VERSION(2, 12, 0)
-		0, basename, 1, doc, 2, color, 3, DOC_FILENAME(doc), -1);
-#else
-		0, basename, 1, doc, 2, color, -1);
-#endif
-	if (! interface_prefs.sidebar_openfiles_fullpath)
-		g_free(basename);
+	{
+		/* path has changed, so remove and re-add */
+		GtkTreeSelection *treesel;
+		gboolean sel;
+
+		treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
+		sel = gtk_tree_selection_iter_is_selected(treesel, &doc->priv->iter);
+		openfiles_remove(doc);
+
+		treeviews_openfiles_add(doc);
+		if (sel)
+			gtk_tree_selection_select_iter(treesel, &doc->priv->iter);
+	}
+	g_free(fname);
 }
 
 
 void treeviews_openfiles_update_all()
 {
-	guint i;
+	guint i, page_count;
 	GeanyDocument *doc;
 
-	gtk_list_store_clear(store_openfiles);
-	for (i = 0; i < (guint) gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)); i++)
+	gtk_tree_store_clear(store_openfiles);
+	page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
+	for (i = 0; i < page_count; i++)
 	{
 		doc = document_get_from_page(i);
 		if (doc == NULL)
@@ -347,9 +405,7 @@ void treeviews_openfiles_update_all()
 
 void treeviews_remove_document(GeanyDocument *doc)
 {
-	GtkTreeIter *iter = &doc->priv->iter;
-
-	gtk_list_store_remove(store_openfiles, iter);
+	openfiles_remove(doc);
 
 	if (GTK_IS_WIDGET(doc->priv->tag_tree))
 	{
@@ -364,54 +420,59 @@ void treeviews_remove_document(GeanyDocument *doc)
 }
 
 
-static void create_taglist_popup_menu(void)
+static void on_hide_sidebar(void)
+{
+	ui_prefs.sidebar_visible = FALSE;
+	ui_sidebar_show_hide();
+}
+
+
+static gboolean on_sidebar_display_symbol_list_show(GtkWidget *item)
+{
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
+		interface_prefs.sidebar_symbol_visible);
+	return FALSE;
+}
+
+
+static gboolean on_sidebar_display_open_files_show(GtkWidget *item)
+{
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
+		interface_prefs.sidebar_openfiles_visible);
+	return FALSE;
+}
+
+
+void sidebar_add_common_menu_items(GtkMenu *menu)
 {
 	GtkWidget *item;
 
-	tv.popup_taglist = gtk_menu_new();
-
-	item = gtk_menu_item_new_with_mnemonic(_("Sort by _Name"));
-	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_taglist), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_taglist_tree_popup_clicked),
-			GINT_TO_POINTER(SYMBOL_ACTION_SORT_BY_NAME));
-
-	item = gtk_menu_item_new_with_mnemonic(_("Sort by _Appearance"));
-	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_taglist), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_taglist_tree_popup_clicked),
-			GINT_TO_POINTER(SYMBOL_ACTION_SORT_BY_APPEARANCE));
-
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_taglist), item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
 
-	mi.symbols_show_symbols = gtk_check_menu_item_new_with_mnemonic(_("Show S_ymbol List"));
-	gtk_widget_show(mi.symbols_show_symbols);
-	gtk_container_add(GTK_CONTAINER(tv.popup_taglist), mi.symbols_show_symbols);
-	g_signal_connect(mi.symbols_show_symbols, "activate",
+	item = gtk_check_menu_item_new_with_mnemonic(_("Show S_ymbol List"));
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "expose-event",
+			G_CALLBACK(on_sidebar_display_symbol_list_show), NULL);
+	gtk_widget_show(item);
+	g_signal_connect(item, "activate",
 			G_CALLBACK(on_list_symbol_activate), NULL);
 
-	mi.symbols_show_documents = gtk_check_menu_item_new_with_mnemonic(_("Show _Document List"));
-	gtk_widget_show(mi.symbols_show_documents);
-	gtk_container_add(GTK_CONTAINER(tv.popup_taglist), mi.symbols_show_documents);
-	g_signal_connect(mi.symbols_show_documents, "activate",
+	item = gtk_check_menu_item_new_with_mnemonic(_("Show _Document List"));
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "expose-event",
+			G_CALLBACK(on_sidebar_display_open_files_show), NULL);
+	gtk_widget_show(item);
+	g_signal_connect(item, "activate",
 			G_CALLBACK(on_list_document_activate), NULL);
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("H_ide Sidebar"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 		gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_MENU));
 	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_taglist), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_taglist_tree_popup_clicked),
-			GINT_TO_POINTER(SYMBOL_ACTION_HIDE_ALL));
-}
-
-
-static void on_openfiles_fullpath_activate(GtkCheckMenuItem *item, gpointer user_data)
-{
-	interface_prefs.sidebar_openfiles_fullpath = gtk_check_menu_item_get_active(item);
-	treeviews_openfiles_update_all();
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_hide_sidebar), NULL);
 }
 
 
@@ -440,6 +501,7 @@ static void create_openfiles_popup_menu(void)
 	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), item);
 	g_signal_connect(item, "activate",
 			G_CALLBACK(on_openfiles_document_action), GINT_TO_POINTER(OPENFILES_ACTION_REMOVE));
+	doc_items.close = item;
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -450,6 +512,7 @@ static void create_openfiles_popup_menu(void)
 	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), item);
 	g_signal_connect(item, "activate",
 			G_CALLBACK(on_openfiles_document_action), GINT_TO_POINTER(OPENFILES_ACTION_SAVE));
+	doc_items.save = item;
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Reload"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
@@ -458,39 +521,22 @@ static void create_openfiles_popup_menu(void)
 	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), item);
 	g_signal_connect(item, "activate",
 			G_CALLBACK(on_openfiles_document_action), GINT_TO_POINTER(OPENFILES_ACTION_RELOAD));
+	doc_items.reload = item;
 
-	item = gtk_separator_menu_item_new();
-	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), item);
+	sidebar_add_common_menu_items(GTK_MENU(tv.popup_openfiles));
+}
 
-	mi.documents_fullpath = gtk_check_menu_item_new_with_mnemonic(_("Show _Full Path Name"));
-	gtk_widget_show(mi.documents_fullpath);
-	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), mi.documents_fullpath);
-	g_signal_connect(mi.documents_fullpath, "activate",
-			G_CALLBACK(on_openfiles_fullpath_activate), NULL);
 
-	item = gtk_separator_menu_item_new();
-	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), item);
+static void unfold_parent(GtkTreeIter *iter)
+{
+	GtkTreeIter parent;
+	GtkTreePath *path;
 
-	mi.documents_show_symbols = gtk_check_menu_item_new_with_mnemonic(_("Show S_ymbol List"));
-	gtk_widget_show(mi.documents_show_symbols);
-	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), mi.documents_show_symbols);
-	g_signal_connect(mi.documents_show_symbols, "activate",
-			G_CALLBACK(on_list_symbol_activate), NULL);
+	gtk_tree_model_iter_parent(GTK_TREE_MODEL(store_openfiles), &parent, iter);
 
-	mi.documents_show_documents = gtk_check_menu_item_new_with_mnemonic(_("Show _Document List"));
-	gtk_widget_show(mi.documents_show_documents);
-	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), mi.documents_show_documents);
-	g_signal_connect(mi.documents_show_documents, "activate",
-			G_CALLBACK(on_list_document_activate), NULL);
-
-	item = gtk_image_menu_item_new_with_mnemonic(_("H_ide Sidebar"));
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-		gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_MENU));
-	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(tv.popup_openfiles), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_openfiles_hide_item_clicked), NULL);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(store_openfiles), &parent);
+	gtk_tree_view_expand_row(GTK_TREE_VIEW(tv.tree_openfiles), path, TRUE);
+	gtk_tree_path_free(path);
 }
 
 
@@ -502,10 +548,13 @@ static gboolean tree_model_find_node(GtkTreeModel *model, GtkTreePath *path,
 {
 	GeanyDocument *doc;
 
-	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, 1, &doc, -1);
+	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, DOCUMENTS_DOCUMENT, &doc, -1);
 
 	if (doc == data)
 	{
+		/* unfolding also prevents a strange bug where the selection gets stuck on the parent
+		 * when it is collapsed and then switching documents */
+		unfold_parent(iter);
 		gtk_tree_view_set_cursor(GTK_TREE_VIEW(tv.tree_openfiles), path, NULL, FALSE);
 		return TRUE;
 	}
@@ -521,48 +570,66 @@ void treeviews_select_openfiles_item(GeanyDocument *doc)
 
 /* callbacks */
 
+static void document_action(GeanyDocument *doc, gint action)
+{
+	if (!DOC_VALID(doc))
+		return;
+
+	switch (action)
+	{
+		case OPENFILES_ACTION_REMOVE:
+		{
+			document_close(doc);
+			break;
+		}
+		case OPENFILES_ACTION_SAVE:
+		{
+			document_save_file(doc, FALSE);
+			break;
+		}
+		case OPENFILES_ACTION_RELOAD:
+		{
+			on_toolbutton_reload_clicked(NULL, NULL);
+			break;
+		}
+	}
+
+}
+
+
 static void on_openfiles_document_action(GtkMenuItem *menuitem, gpointer user_data)
 {
 	GtkTreeIter iter;
 	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
 	GtkTreeModel *model;
 	GeanyDocument *doc;
+	gint action = GPOINTER_TO_INT(user_data);
 
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
-		gtk_tree_model_get(model, &iter, 1, &doc, -1);
-		if (DOC_VALID(doc))
+		gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc, -1);
+		if (doc)
 		{
-			switch (GPOINTER_TO_INT(user_data))
+			document_action(doc, action);
+		}
+		else
+		{
+			/* parent item selected */
+			GtkTreeIter child;
+			gint i = gtk_tree_model_iter_n_children(model, &iter) - 1;
+
+			while (i >= 0 && gtk_tree_model_iter_nth_child(model, &child, &iter, i))
 			{
-				case OPENFILES_ACTION_REMOVE:
-				{
-					document_close(doc);
-					break;
-				}
-				case OPENFILES_ACTION_SAVE:
-				{
-					document_save_file(doc, FALSE);
-					break;
-				}
-				case OPENFILES_ACTION_RELOAD:
-				{
-					on_toolbutton_reload_clicked(NULL, NULL);
-					break;
-				}
+				gtk_tree_model_get(model, &child, DOCUMENTS_DOCUMENT, &doc, -1);
+
+				document_action(doc, action);
+				i--;
 			}
 		}
 	}
 }
 
 
-static void on_openfiles_hide_item_clicked(GtkMenuItem *menuitem, gpointer user_data)
-{
-	ui_prefs.sidebar_visible = FALSE;
-	ui_sidebar_show_hide();
-}
-
-
 static gboolean change_focus(gpointer data)
 {
 	GeanyDocument *doc = data;
@@ -589,7 +656,9 @@ static void on_openfiles_tree_selection_changed(GtkTreeSelection *selection, gpo
 	/* use switch_notebook_page to ignore changing the notebook page because it is already done */
 	if (gtk_tree_selection_get_selected(selection, &model, &iter) && ! ignore_callback)
 	{
-		gtk_tree_model_get(model, &iter, 1, &doc, -1);
+		gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc, -1);
+		if (!doc)
+			return;	/* parent */
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
 					gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
 					(GtkWidget*) doc->editor->sci));
@@ -598,40 +667,6 @@ static void on_openfiles_tree_selection_changed(GtkTreeSelection *selection, gpo
 }
 
 
-static void on_taglist_tree_popup_clicked(GtkMenuItem *menuitem, gpointer user_data)
-{
-	switch (GPOINTER_TO_INT(user_data))
-	{
-		case SYMBOL_ACTION_SORT_BY_NAME:
-		{
-			GeanyDocument *doc = document_get_current();
-			if (doc != NULL)
-				doc->has_tags = symbols_recreate_tag_list(doc, SYMBOLS_SORT_BY_NAME);
-			break;
-		}
-		case SYMBOL_ACTION_SORT_BY_APPEARANCE:
-		{
-			GeanyDocument *doc = document_get_current();
-			if (doc != NULL)
-				doc->has_tags = symbols_recreate_tag_list(doc, SYMBOLS_SORT_BY_APPEARANCE);
-			break;
-		}
-		case SYMBOL_ACTION_HIDE:
-		{
-			interface_prefs.sidebar_symbol_visible = FALSE;
-			ui_sidebar_show_hide();
-			break;
-		}
-		case SYMBOL_ACTION_HIDE_ALL:
-		{
-			ui_prefs.sidebar_visible = FALSE;
-			ui_sidebar_show_hide();
-			break;
-		}
-	}
-}
-
-
 static gboolean on_taglist_tree_selection_changed(GtkTreeSelection *selection)
 {
 	GtkTreeIter iter;
@@ -640,7 +675,13 @@ static gboolean on_taglist_tree_selection_changed(GtkTreeSelection *selection)
 
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
-		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_LINE, &line, -1);
+		const TMTag *tag;
+
+		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_TAG, &tag, -1);
+		if (!tag)
+			return FALSE;
+
+		line = tag->atts.entry.line;
 		if (line > 0)
 		{
 			GeanyDocument *doc = document_get_current();
@@ -669,16 +710,18 @@ static gboolean on_treeviews_key_press_event(GtkWidget *widget, GdkEventKey *eve
 }
 
 
-static gboolean on_treeviews_button_press_event(GtkWidget *widget, GdkEventButton *event,
-												gpointer user_data)
+static gboolean on_symbols_button_press_event(GtkWidget *widget, GdkEventButton *event,
+		G_GNUC_UNUSED gpointer user_data)
 {
-	if (event->type == GDK_2BUTTON_PRESS && GPOINTER_TO_INT(user_data) == TREEVIEW_SYMBOL)
+	GtkTreeSelection *selection;
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
+
+	if (event->type == GDK_2BUTTON_PRESS)
 	{	/* double click on parent node(section) expands/collapses it */
 		GtkTreeModel *model;
-		GtkTreeSelection *selection;
 		GtkTreeIter iter;
 
-		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 		if (gtk_tree_selection_get_selected(selection, &model, &iter))
 		{
 			if (gtk_tree_model_iter_has_child(model, &iter))
@@ -695,36 +738,59 @@ static gboolean on_treeviews_button_press_event(GtkWidget *widget, GdkEventButto
 			}
 		}
 	}
-	else if (event->button == 1 && GPOINTER_TO_INT(user_data) == TREEVIEW_SYMBOL)
+	else if (event->button == 1)
 	{	/* allow reclicking of taglist treeview item */
-		GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 		/* delay the query of selection state because this callback is executed before GTK
 		 * changes the selection (g_signal_connect_after would be better but it doesn't work) */
-		g_idle_add((GSourceFunc) on_taglist_tree_selection_changed, select);
+		g_idle_add((GSourceFunc) on_taglist_tree_selection_changed, selection);
 	}
 	else if (event->button == 3)
-	{	/* popupmenu to hide or clear the active treeview */
-		if (GPOINTER_TO_INT(user_data) == TREEVIEW_OPENFILES)
-		{
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi.documents_show_documents),
-				interface_prefs.sidebar_openfiles_visible);
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi.documents_show_symbols),
-				interface_prefs.sidebar_symbol_visible);
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi.documents_fullpath),
-				interface_prefs.sidebar_openfiles_fullpath);
-			gtk_menu_popup(GTK_MENU(tv.popup_openfiles), NULL, NULL, NULL, NULL,
-																event->button, event->time);
-		}
-		else if (GPOINTER_TO_INT(user_data) == TREEVIEW_SYMBOL)
-		{
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi.symbols_show_documents),
-				interface_prefs.sidebar_openfiles_visible);
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi.symbols_show_symbols),
-				interface_prefs.sidebar_symbol_visible);
-			gtk_menu_popup(GTK_MENU(tv.popup_taglist), NULL, NULL, NULL, NULL,
-																event->button, event->time);
-			return TRUE;	/* prevent selection changed signal for symbol tags */
-		}
+	{
+		gtk_menu_popup(GTK_MENU(tv.popup_taglist), NULL, NULL, NULL, NULL,
+															event->button, event->time);
+		return TRUE;	/* prevent selection changed signal for symbol tags */
+	}
+	return FALSE;
+}
+
+
+static void documents_menu_update(GtkTreeSelection *selection)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean sel, path;
+	gchar *shortname = NULL;
+	GeanyDocument *doc = NULL;
+
+	/* maybe no selection e.g. if ctrl-click deselected */
+	sel = gtk_tree_selection_get_selected(selection, &model, &iter);
+	if (sel)
+	{
+		gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc,
+			DOCUMENTS_SHORTNAME, &shortname, -1);
+	}
+	path = NZV(shortname) && g_path_is_absolute(shortname);
+
+	/* can close all, save all (except shortname), but only reload individually ATM */
+	gtk_widget_set_sensitive(doc_items.close, sel);
+	gtk_widget_set_sensitive(doc_items.save, (doc && doc->real_path) || path);
+	gtk_widget_set_sensitive(doc_items.reload, doc && doc->real_path);
+	g_free(shortname);
+}
+
+
+static gboolean on_documents_button_release_event(GtkWidget *widget, GdkEventButton *event,
+												gpointer user_data)
+{
+	GtkTreeSelection *selection;
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
+
+	if (event->button == 3)
+	{
+		documents_menu_update(selection);
+		gtk_menu_popup(GTK_MENU(tv.popup_openfiles), NULL, NULL, NULL, NULL,
+			event->button, event->time);
 	}
 	return FALSE;
 }
@@ -733,10 +799,9 @@ static gboolean on_treeviews_button_press_event(GtkWidget *widget, GdkEventButto
 void treeviews_init()
 {
 	tv.default_tag_tree = NULL;
-	tag_window = lookup_widget(main_widgets.window, "scrolledwindow2");
+	tag_window = ui_lookup_widget(main_widgets.window, "scrolledwindow2");
 
 	prepare_openfiles();
-	create_taglist_popup_menu();
 	create_openfiles_popup_menu();
 }
 
